% factor model identification according to identified factor patterns (run identify_factors_new before)
% 1. correlate the draws with the identified factor pattern, permute accordingly factor-specific parameters and perform sign-identification
%    (i) keep those draws in it_sel.in which are uniquely assigned to the factor patterns based on maximum correlation with factor patterns
%    (ii) permute all factor-specific (loadings etc.) parameters according to the assignment of the factor draws to the factor patterns
%    (iii) switch the sign of factor draws and factor-specific parameters which are negatively correlated with the assigned factor patterns
% 2. perform overall sign switch, to achieve a majority of positive median factor loadings
% 3. rank the factors in descending order of the number of non-zero factor loading or in %descending order or exclusive loadings (currently implemented)

nf=size(ftmcstar,2);
MMC=size(ftmcstar,3);

factors=(1:nf);
it_sel=struct([]);
it_sel(1).out=[];
it_sel(1).in=[];
nft=size(ftstmed,2);
nm=nft;
perm_pat=[];

% 1. correlate the draws with the identified factor pattern, permute accordingly factor-specific parameters and perform sign-identification
for mc=1:MMC;
    %     lamstarmc(:,:,mc)=full(Mc.lamsp(mc).draw);
    co_mat=corr(ftstmed,ftmcstar(:,:,mc));
    co_max=max(abs(co_mat),[],1);
    ind_co=(abs(co_mat)==kron(ones(nft,1),co_max));
    [si,r]=find(ind_co');
    co_sign=sign(co_mat(ind_co));
    co_sign=co_sign(si);
    ind_sig=find(co_sign<0);
    
    %    (i) keep those draws in it_sel.in which are uniquely assigned to the factor patterns based on maximum correlation with factor patterns
    if any([sum(ind_co,1) sum(ind_co,2)']>1)
        it_sel(1).out(end+1)=mc;
    else
        it_sel(1).in(end+1)=mc;
        %    (ii) permute all factor-specific (loadings etc.) parameters according to the assignment of the factor draws to the factor patterns
        lamstarmc(:,:,mc)=lamstarmc(:,si,mc);
        ftmcstar(:,:,mc)=ftmcstar(:,si,mc);
        for l=1:philag
            phistarmc(:,(1:nf)+(l-1)*nf,mc)=phistarmc(si,si+(l-1)*nf,mc);
        end
        if prior==2
            betamc(:,:,mc)=betamc(:,si,mc);
        end
        if prior
            rhomc(:,1,mc)=rhomc(si,1,mc);
        end
        
        taumc(:,1,mc)=taumc(si,1,mc);
        %    (iii) switch the sign of factor draws and factor-specific parameters which are negatively correlated with the assigned factor patterns
        if any(ind_sig)
            nk=find(~sum(factors(ones(1,length(ind_sig)),:)==ind_sig(:,ones(1,nf)),1));
            lamstarmc(:,ind_sig,mc)=-lamstarmc(:,ind_sig,mc);
            ftmcstar(:,ind_sig,mc)=-ftmcstar(:,ind_sig,mc);
            for l=1:philag
                phistarmc(nk,ind_sig+(l-1)*nf,mc)=-phistarmc(nk,ind_sig+(l-1)*nf,mc);
                phistarmc(ind_sig,nk+(l-1)*nf,mc)=-phistarmc(ind_sig,nk+(l-1)*nf,mc);
            end
        end
    end
    
end

% 2. perform overall sign switch, to achieve a majority of positive factor loadings
if prior
    lamstarm=median(lamstarmc(:,:,it_sel.in).*(betamc(:,:,it_sel.in)>0.5),3);
else
    lamstarm=median(lamstarmc(:,:,it_sel.in),3);
end
s_ind=checkpositivity(lamstarm);
if any(s_ind)
    nk=find(~any(factors(ones(1,length(s_ind)),:)'==s_ind(ones(1,nf),:),2));
    lamstarmc(:,s_ind,it_sel(1).in)=-lamstarmc(:,s_ind,it_sel(1).in);
    ftmcstar(:,s_ind,it_sel(1).in)=-ftmcstar(:,s_ind,it_sel(1).in);
    for l=1:philag
        phistarmc(nk,s_ind+(l-1)*nf,it_sel(1).in)=-phistarmc(nk,s_ind+(l-1)*nf,it_sel(1).in);
        phistarmc(s_ind,nk+(l-1)*nf,it_sel(1).in)=-phistarmc(s_ind,nk+(l-1)*nf,it_sel(1).in);
    end
end


% 3. rank the factors in descending order of exclusive loadings or the number of non-zero factor loading (commented out)
if prior>0
    %     [lao,si]=sort(sum(lamstarm~=0),'descend');
    lamstarm=lamstarm~=0;
    excl_load=zeros(1,nf);
    for j=1:nf
        excl_load(j)=sum((lamstarm(:,j)-any(lamstarm(:,[1:j-1 j+1:nf]),2))>0);
    end
    [excl_load,si]=sort(excl_load,'descend');
    
    lamstarmc=lamstarmc(:,si,:);
    ftmcstar=ftmcstar(:,si,:);
    for l=1:philag
        phistarmc(:,(1:nf)+(l-1)*nf,:)=phistarmc(si,si+(l-1)*nf,:);
    end
    if any(betamc)
        betamc=betamc(:,si,:);
    end
    if any(rhomc)
        rhomc=rhomc(si,1,:);
    end
    taumc(:,1,mc)=taumc(si,1,mc);
end
