function [prminn] = minnesota(y,X,lags,minn_var,minn_cov);

%Minnesota prior for Bayesian VAR analysis
%y  TxN   matrix of variables (T observations on N variables)
%X  TxN*lags matrix containing lagged y [y_(t-1)...y(t-lags)]
%lags     number of AR lags
%minn_var prior variance for the first autoregressive lag
%minn_cov shrink factor for prior variance on lags of other variables

%output
%prminn  NxN*lags [A_1 ... A_lags] matrix containing prior information of coefficients 
%       (element ij of A_q gives the effect of variable j on variable i at lag q)

nvars=size(y,2);
T=size(y,1);
std_res=ones(nvars,1);
if any(y)
    %compute std of resid of OLS-regression of y on own lags
    Y=reshape(y',T*nvars,1);
    Xmat= (kron(X,ones(nvars,1))).* kron(ones(T,1),kron(ones(1,lags),eye(nvars)));
    res=Y-Xmat/(Xmat'*Xmat)*Xmat'*Y;
    % res=(eye(nvars*T)-Xmat*inv(Xmat'*Xmat)*Xmat')*Y;

    std_res = sqrt(diag(reshape(res,nvars,T)*reshape(res,nvars,T)')./T);
end
scale_mat = kron(ones(1,lags),std_res(:,ones(1,nvars))./std_res(:,ones(1,nvars))');

var_mat = sqrt(minn_var)./kron(1:lags,ones(nvars));
wei_mat = sqrt(minn_cov)*(ones(nvars,nvars*lags)- kron(ones(1,lags),eye(nvars))) + kron(ones(1,lags),eye(nvars));

prminn = (wei_mat.*var_mat.*scale_mat).^(-2);